@@ -9,8 +9,10 @@
 #
 import re
 import math
-from .helpers import collapse_white_spaces
-from .atomic_types import AtomicTypeABCMeta, AnyAtomicType
+from typing import Optional
+
+from ..helpers import collapse_white_spaces
+from .atomic_types import AtomicTypeMeta, AnyAtomicType
 
 
 class Float10(float, AnyAtomicType):
@@ -29,8 +31,12 @@ class Float10(float, AnyAtomicType):
                 raise ValueError('invalid value {!r} for xs:{}'.format(value, cls.name))
 
         value = super().__new__(cls, value)
-        if -1e-37 < value < 1e-37:
-            return super().__new__(cls, 0.0)
+        if value > 3.4028235E38:
+            return super().__new__(cls, 'INF')
+        elif value < -3.4028235E38:
+            return super().__new__(cls, '-INF')
+        elif -1e-37 < value < 1e-37:
+            return super().__new__(cls, -0.0 if str(value).startswith('-') else 0.0)
         return value
 
     def __hash__(self):
@@ -109,11 +115,12 @@ class Float(Float10):
     xsd_version = '1.1'
 
 
-class Integer(int, metaclass=AtomicTypeABCMeta):
+class Integer(int, metaclass=AtomicTypeMeta):
     """A wrapper for emulating xs:integer and limited integer types."""
     name = 'integer'
     pattern = re.compile(r'^[\-+]?[0-9]+$')
-    lower_bound, higher_bound = None, None
+    lower_bound: Optional[int] = None
+    higher_bound: Optional[int] = None
 
     def __init__(self, value):
         if self.lower_bound is not None and self < self.lower_bound:
@@ -170,7 +177,8 @@ class Byte(Short):
 
 class NonNegativeInteger(Integer):
     name = 'nonNegativeInteger'
-    lower_bound, higher_bound = 0, None
+    lower_bound = 0
+    higher_bound: Optional[int] = None
 
 
 class PositiveInteger(NonNegativeInteger):
