@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2018-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2018-2021, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -8,16 +8,28 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import datetime
+import importlib
+from copy import copy
 from functools import lru_cache
 from itertools import chain
+from types import ModuleType
+from typing import TYPE_CHECKING, cast, Dict, Any, List, Iterable, Iterator, Optional, Union
 
 from .exceptions import ElementPathTypeError
-from .datatypes import Timezone
+from .datatypes import AnyAtomicType, Timezone
 from .xpath_nodes import TypedElement, AttributeNode, TextNode, TypedAttribute, \
-    etree_iter_nodes, is_etree_element, is_element_node, is_document_node, is_schema_node
+    etree_iter_nodes, is_etree_element, is_element_node, is_document_node, \
+    is_schema_node, is_lxml_etree_element, is_lxml_document_node, XPathNode, \
+    ElementNode, DocumentNode
+
+if TYPE_CHECKING:
+    from .xpath_token import XPathToken, XPathAxis
+
+ContextRootType = Union[ElementNode, DocumentNode]
+ContextItemType = Optional[Union[ElementNode, DocumentNode, XPathNode, AnyAtomicType]]
 
 
-class XPathContext(object):
+class XPathContext:
     """
     The XPath dynamic context. The static context is provided by the parser.
 
@@ -40,30 +52,51 @@ class XPathContext(object):
     :param current_dt: current dateTime of the implementation, including explicit timezone.
     :param timezone: implicit timezone to be used when a date, time, or dateTime value does \
     not have a timezone.
-    :param documents: available documents. This is a dictionary from absolute URIs \
-    onto document nodes. Used by the function fn:doc.
-    :param collections: available collections. This is a dictionary from absolute \
-    URIs onto sequences of nodes. Used by the function fn:collection.
+    :param documents: available documents. This is a mapping of absolute URI \
+    strings onto document nodes. Used by the function fn:doc.
+    :param collections: available collections. This is a mapping of absolute URI \
+    strings onto sequences of nodes. Used by the XPath 2.0+ function fn:collection.
     :param default_collection: this is the sequence of nodes used when fn:collection \
     is called with no arguments.
+    :param resource_collections: available URI collections. This is a mapping of absolute \
+    URI strings to sequences of URIs. Used by the XPath 3.0+ function fn:uri-collection.
+    :param default_resource_collection: this is the sequence of URIs used when \
+    fn:uri-collection is called with no arguments.
+    :param allow_environment: defines if the access to system environment is allowed, \
+    for default is `False`. Used by the XPath 3.0+ functions fn:environment-variable \
+    and fn:available-environment-variables.
     """
     _iter_nodes = staticmethod(etree_iter_nodes)
-    _parent_map = None
-    _elem = None
+    _parent_map: Optional[Dict[ElementNode, ContextRootType]] = None
+    _etree: Optional[ModuleType] = None
+    root: ContextRootType
+    item: Optional[ContextItemType]
+
+    def __init__(self,
+                 root: ContextRootType,
+                 namespaces: Optional[Dict[str, str]] = None,
+                 item: Optional[ContextItemType] = None,
+                 position: int = 1,
+                 size: int = 1,
+                 axis: Optional[str] = None,
+                 variables: Optional[Dict[str, Any]] = None,
+                 current_dt: Optional[datetime.datetime] = None,
+                 timezone: Optional[Timezone] = None,
+                 documents: Optional[DocumentNode] = None,
+                 collections: Optional[ElementNode] = None,
+                 default_collection: Optional[str] = None,
+                 resource_collections: Optional[Dict[str, List[str]]] = None,
+                 default_resource_collection: Optional[str] = None,
+                 allow_environment: bool = False,
+                 default_language: Optional[str] = None,
+                 default_calendar: Optional[str] = None,
+                 default_place=None):
 
-    def __init__(self, root, namespaces=None, item=None, position=1, size=1, axis=None,
-                 variables=None, current_dt=None, timezone=None, documents=None,
-                 collections=None, default_collection=None):
         self.root = root
         self.namespaces = namespaces
 
-        if is_etree_element(root) and not callable(root.tag):
-            if item is None:
-                self.item = self._elem = root
-            elif is_etree_element(item) and not callable(root.tag):
-                self.item = self._elem = item
-            else:
-                self.item, self._elem = item, root
+        if is_element_node(root):
+            self.item = root if item is None else item
         elif is_document_node(root):
             self.item = item
         else:
@@ -88,47 +121,54 @@ class XPathContext(object):
         self.documents = documents
         self.collections = collections
         self.default_collection = default_collection
+        self.resource_collections = resource_collections
+        self.default_resource_collection = default_resource_collection
+        self.allow_environment = allow_environment
+        self.default_language = default_language
+        self.default_calendar = default_calendar
+        self.default_place = default_place
 
     def __repr__(self):
         return '%s(root=%r, item=%r)' % (self.__class__.__name__, self.root, self.item)
 
     def __copy__(self):
-        obj = type(self)(
-            root=self.root,
-            namespaces=self.namespaces,
-            item=self.item,
-            position=self.position,
-            size=self.size,
-            axis=None,
-            variables=self.variables,
-            current_dt=self.current_dt,
-            timezone=self.timezone,
-            documents=self.documents,
-            collections=self.collections,
-            default_collection=self.default_collection,
-        )
-        if self.item is None:
-            obj.item = None
-        obj._elem = self._elem
-        obj._parent_map = self._parent_map
+        obj = object.__new__(self.__class__)
+        obj.__dict__.update(self.__dict__)
+        obj.axis = None
+        obj.variables = {k: v for k, v in self.variables.items()}
         return obj
 
     def copy(self, clear_axis=True):
+        # Unused, so it could be deprecated in the future.
+        obj = object.__new__(self.__class__)
+        obj.__dict__.update(self.__dict__)
         if clear_axis:
-            return self.__copy__()
-        else:
-            obj = self.__copy__()
-            obj.axis = self.axis
-            return obj
+            obj.axis = None
+        obj.variables = {k: v for k, v in self.variables.items()}
+        return obj
 
     @property
-    def parent_map(self):
+    def parent_map(self) -> Dict[ElementNode, ContextRootType]:
         if self._parent_map is None:
+            self._parent_map: Dict[ElementNode, ContextRootType]
             self._parent_map = {child: elem for elem in self.root.iter() for child in elem}
+            if is_document_node(self.root):
+                self._parent_map[cast(DocumentNode, self.root).getroot()] = self.root
+
         return self._parent_map
 
+    @property
+    def etree(self) -> ModuleType:
+        if self._etree is None:
+            self._etree: ModuleType
+            if is_lxml_etree_element(self.root) or is_lxml_document_node(self.root):
+                self._etree = importlib.import_module('lxml.etree')
+            else:
+                self._etree = importlib.import_module('xml.etree.ElementTree')
+        return self._etree
+
     @lru_cache(maxsize=1024)
-    def get_parent(self, elem):
+    def get_parent(self, elem: Union[ElementNode, TypedElement]):
         """
         Returns the parent element or `None` for root element and for elements
         that are not included in the tree. Uses a LRU cache to minimize parent
@@ -137,29 +177,22 @@ class XPathContext(object):
         if isinstance(elem, TypedElement):
             elem = elem.elem
         if elem is self.root:
-            return
+            return None
 
         try:
-            return self._parent_map[elem]
-        except (KeyError, TypeError):
-            self._parent_map = {child: e for e in self.root.iter() for child in e}
-            if is_document_node(self.root):
-                self._parent_map[self.root.getroot()] = self.root
+            return self.parent_map[elem]
+        except KeyError:
+            return None
 
-            try:
-                return self._parent_map[elem]
-            except KeyError:
-                return
-
-    def get_path(self, item):
+    def get_path(self, item: Any) -> str:
         """Cached path resolver for elements and attributes. Returns absolute paths."""
         path = []
         if isinstance(item, AttributeNode):
-            path.append('@%s' % item.name)
-            item = self._elem
+            path.append(f'@{item.name}')
+            item = item.parent
         elif isinstance(item, TypedAttribute):
-            path.append('@%s' % item.attribute.name)
-            item = self._elem
+            path.append(f'@{item.attribute.name}')
+            item = item.attribute.parent
 
         if item is None:
             return '' if not path else path[0]
@@ -183,15 +216,15 @@ class XPathContext(object):
         else:
             return is_element_node(self.item)
 
-    def iter(self):
+    def iter(self) -> Iterator[Union[ElementNode, DocumentNode, AttributeNode, TextNode]]:
         """Iterates context nodes, including text and attribute nodes."""
         root = self.root
         if is_document_node(root):
             yield root
-            root = root.getroot()
+            root = cast(DocumentNode, root).getroot()
         yield from self._iter_nodes(root, with_attributes=True)
 
-    def iter_results(self, results):
+    def iter_results(self, results: Iterable) -> Iterator[ContextItemType]:
         """
         Iterates results in document order.
 
@@ -217,12 +250,12 @@ class XPathContext(object):
 
         self.item = status
 
-    def inner_focus_select(self, token):
+    def inner_focus_select(self, token: Union['XPathToken', 'XPathAxis']):
         """Apply the token's selector with an inner focus."""
         status = self.item, self.size, self.position
-        results = [x for x in token.select(self.copy())]
+        results = [x for x in token.select(copy(self))]
 
-        if token.label == 'axis' and token.reverse_axis:
+        if token.label == 'axis' and cast('XPathAxis', token).reverse_axis:
             self.size = self.position = len(results)
             for self.item in results:
                 yield self.item
@@ -277,8 +310,10 @@ class XPathContext(object):
         yield self.item
         self.axis = status
 
-    def iter_attributes(self):
+    def iter_attributes(self) -> Iterator[Union[AttributeNode, TypedAttribute]]:
         """Iterator for 'attribute' axis and '@' shortcut."""
+        status: Any
+
         if isinstance(self.item, (AttributeNode, TypedAttribute)):
             status = self.axis
             self.axis = 'attribute'
@@ -294,13 +329,13 @@ class XPathContext(object):
         if isinstance(self.item, TypedElement):
             self.item = self.item.elem
 
-        elem = self.item
+        elem = cast(ElementNode, self.item)
         if is_schema_node(elem):
             # TODO: for backward compatibility, to be removed in release 3.0.
             for self.item in (AttributeNode(*x) for x in elem.attrib.items()):
                 yield self.item
         else:
-            for self.item in (AttributeNode(*x, elem) for x in elem.attrib.items()):
+            for self.item in (AttributeNode(x[0], x[1], parent=elem) for x in elem.attrib.items()):
                 yield self.item
 
         self.item, self.axis = status
